/*****************************************************************
 * PTlink IRCd is (C) CopyRight PTlink Coders Team 1999-2001     *
 * http://www.ptlink.net/Coders - coders@PTlink.net              *
 * This program is distributed under GNU Public License          *
 * Please read the file COPYING for copyright information.       *
 *****************************************************************
 
 File: sgline.c
 Desc: services glines (real name glines)
 Author: Lamego@PTlink.net
  
*/

#include "m_commands.h"
#include "client.h"
#include "s_user.h"
#include "s_misc.h"
#include "common.h"
#include "ircd.h"
#include "numeric.h"
#include "s_serv.h"
#include "irc_string.h"
#include "send.h"
#include "s_conf.h"
#include "struct.h"
#include "sgline.h"
#include "conf_vars.h"


#include <stdlib.h> 
#include <string.h>

/* sglines storage */
aConfItem *sglines = (aConfItem *)NULL;

/*
 * m_sgline() - add info ban line
 *
 *	parv[0] = sender prefix
 *	parv[1] = real name banned mask
 */
int	m_sgline(struct Client *cptr, struct Client *sptr, int parc, char *parv[])
  {
    aConfItem *aconf;

	if (!IsService(sptr) && !IsServer(cptr))
	  {
		sendto_one(sptr, form_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
	  }	  
		
	if(parc<2)
	  {
		sendto_one(sptr, form_str(ERR_NEEDMOREPARAMS),
      	  me.name, parv[0], "SGLINE");
        return 0;
	  }
	  
    if (!find_sgline(parv[1])) /* sgline does not exist */
	  {

		aconf = make_conf();
		DupString(aconf->name, parv[1]);

	    aconf->next = sglines;
		sglines = aconf;		

	    sendto_serv_butone(cptr, ":%s SGLINE :%s",
			parv[0], parv[1]);
			
	  }

	return 0;
  }

/*
 * m_unsgline() - remove zap line
 *
 *	parv[0] = sender prefix
 *	parv[1] = sglined name
 */
int	m_unsgline(struct Client *cptr, struct Client *sptr, int parc, char *parv[])
  {
	aConfItem *sgl = sglines;
	aConfItem *last_sgl = NULL;
		
	if (!IsService(sptr) && !IsServer(cptr))
	  {
		sendto_one(sptr, form_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
	  }	  
		
	if(parc<2)
	  {
		sendto_one(sptr, form_str(ERR_NEEDMOREPARAMS),
      	  me.name, parv[0], "UNSGLINE");
        return 0;
	  }		  
	
	while(sgl && irccmp(sgl->name, parv[1]))
	  {
		last_sgl = sgl;
		sgl = sgl->next;
	  }
	  
    if (sgl) /* if sgline does exist */
	  {
		if(last_sgl) /* this is not the first sgline */
			last_sgl->next=sgl->next; /* adjust list link -> */
		  else
			sglines = sgl->next;
			
		free_conf(sgl);
		
	    sendto_serv_butone(cptr, ":%s UNSGLINE %s",
			parv[0], parv[1]);			
	  }
	  
	return 0;
  }
  

/*
 * find_sgline  - checks if the host matchs any entry on the sqline list
 * inputs       - lookup host
 * output       - return sgline entry if found or NULL if not found;
 * side effects - heading sgline will be checked and deleted if expired
 */
struct ConfItem* find_sgline(char *name)
  {
	aConfItem *sgl;

	/* First lets check if we have sglines */
	if(!sglines)
	  return NULL;
	  	  
	sgl = sglines;

    /* search for host */	
	while(sgl && irccmp(sgl->name, name))
	  sgl=sgl->next;
	  
    return sgl;	
		
  }

/*
 * is_sglined  - checks if the host matchs any entry on the sqline list
 * inputs       - lookup name
 * output       - return sgline entry if found or NULL if not found;
 * side effects - heading sgline will be checked and deleted if expired
 */
struct ConfItem* is_sglined(char *name)
  {
	aConfItem *sgl;

	/* First lets check if we have sglines */
	if(!sglines)
	  return NULL;
	  	  
	sgl = sglines;

    /* search for host */	
	while(sgl && !match(sgl->name, name))
	  sgl=sgl->next;
	  
    return sgl;	
		
  }
  
  
/* send_all_sglines
 *
 * inputs       - pointer to aClient
 * output       - none
 * Side effects - sends all sglines to the specified server
 *
 */
void send_all_sglines(struct Client *acptr)
  {
	struct ConfItem *sgl = sglines;
	
	while(sgl)
	  {	  
	  	  
  		  sendto_one(acptr, ":%s SGLINE :%s",
			  me.name, sgl->name);
			  		  
		sgl = sgl->next;		
	  }
  }
  
/*
 * report_sglines
 *
 * inputs       - pointer to client to report to
 * output       - none
 * side effects - all Z lines are listed to client 
 */
void report_sglines(struct Client *sptr)
{
  struct ConfItem *aconf;
  char *host;
  char *user;
  char *pass;
  char *name;
  int port;
  
  aconf = sglines;
  
  while(aconf)
	{
  	  get_printable_conf(aconf, &name, &host, &pass, &user, &port);
          
      sendto_one(sptr, form_str(RPL_STATSQLINE),
                   me.name, sptr->name, name, pass, user, host);
	  aconf=aconf->next;	  
    }
}

/*
 * clear_sglines
 *
 * inputs       - none
 * output       - none
 * side effects - clear the sgline list
 */
void clear_sglines(void)
  {
  
    aConfItem *aconf;
	aConfItem *sgl=sglines;
  
	while(sgl)
	  {
		aconf = sgl->next;
		free_conf(sgl);
		sgl = aconf;
		}
			  
	sglines = NULL;
	
  }
